part of 'pagination_bloc.dart';

abstract class PaginationState extends Equatable {
  const PaginationState();

  @override
  List<Object?> get props => [];
}

class PaginationNotInitial extends PaginationState {}

class PaginationInitial extends PaginationState {}

class PaginationFailure extends PaginationState {
  final String error;
  const PaginationFailure(this.error);
  @override
  String toString() => 'PaginationFailure $error';
}

class PaginationSuccess<T> extends PaginationState {
  final List<T> data;
  final bool hasReachedMax;
  final DocumentSnapshot<Map<String, dynamic>>? lastSnapshot;
  const PaginationSuccess({
    required this.data,
    this.lastSnapshot,
    this.hasReachedMax = false,
  });

  PaginationSuccess<T> copyWith({
    List<T>? data,
    bool? hasReachedMax,
    DocumentSnapshot<Map<String, dynamic>>? lastSnapshot,
  }) {
    return PaginationSuccess<T>(
      data: data ?? this.data,
      hasReachedMax: hasReachedMax ?? this.hasReachedMax,
      lastSnapshot: lastSnapshot ?? this.lastSnapshot,
    );
  }

  @override
  List<Object?> get props => [data, hasReachedMax, lastSnapshot];

  @override
  String toString() => 'PaginationSuccess { data: ${data.length}, hasReachedMax: $hasReachedMax }';
}

class PaginationUpdate<T> extends PaginationSuccess<T> {
  PaginationUpdate({
    required List<T> data,
    DocumentSnapshot<Map<String, dynamic>>? lastSnapshot,
    bool hasReachedMax = false,
  }) : super(data: data, lastSnapshot: lastSnapshot, hasReachedMax: hasReachedMax) {
    this.updateTime = DateTime.now();
  }

  factory PaginationUpdate.fromSuccess(PaginationSuccess<T> success) {
    return PaginationUpdate(
      data: success.data,
      lastSnapshot: success.lastSnapshot,
      hasReachedMax: success.hasReachedMax,
    );
  }
  late final DateTime updateTime;
  PaginationUpdate<T> copyWith({
    List<T>? data,
    bool? hasReachedMax,
    DocumentSnapshot<Map<String, dynamic>>? lastSnapshot,
  }) {
    return PaginationUpdate<T>(
      data: data ?? this.data,
      hasReachedMax: hasReachedMax ?? this.hasReachedMax,
      lastSnapshot: lastSnapshot ?? this.lastSnapshot,
    );
  }

  @override
  List<Object?> get props => [updateTime, data, hasReachedMax, lastSnapshot];

  @override
  String toString() => 'PaginationUpdate { data: ${data.length} }';
}
