import 'package:firestore_repository/firestore_repository.dart';
import 'package:flutter/material.dart';

class PaginationArgs<T> {
  PaginationArgs({
    required this.query,
    required this.loadingWidget,
    required this.converter,
    this.limit = 20,
    this.maxReachedWidget,
    this.scrollController,
    this.onInitial,
    this.onFailure,
    this.onSuccess,
    this.onEmpty,
    this.buildList,
    this.buildItem,
    this.customBuilder,
    this.physics,
    this.shrinkWrap = false,
    this.reverse = false,
  });

  final Converter<T> converter;
  final int limit;
  final Query<Map<String, Object?>> query;

  final StateChange<PaginationInitial>? onInitial;
  final StateChange<PaginationFailure>? onFailure;
  final StateChange<PaginationSuccess<T>>? onSuccess;
  final StateChange<PaginationSuccess<T>>? onEmpty;
  final StateChange<PaginationSuccess<T>>? buildList;
  final StateChange<PaginationState>? customBuilder;
  final PaginationItemBuilder<T>? buildItem;
  final Widget? loadingWidget;
  final Widget? maxReachedWidget;
  final ScrollController? scrollController;
  final ScrollPhysics? physics;
  final bool shrinkWrap;
  final bool reverse;

  PaginationArgs<T> copyWith({
    Converter<T>? converter,
    int? limit,
    Query<Map<String, Object?>>? query,
    StateChange<PaginationInitial>? onInitial,
    StateChange<PaginationFailure>? onFailure,
    StateChange<PaginationSuccess<T>>? onSuccess,
    StateChange<PaginationSuccess<T>>? onEmpty,
    StateChange<PaginationSuccess<T>>? buildList,
    StateChange<PaginationState>? customBuilder,
    PaginationItemBuilder<T>? buildItem,
    Widget? loadingWidget,
    Widget? maxReachedWidget,
    ScrollController? scrollController,
    ScrollPhysics? physics,
    bool shrinkWrap = false,
    bool reverse = false,
  }) {
    return PaginationArgs<T>(
      converter: converter ?? this.converter,
      limit: limit ?? this.limit,
      query: query ?? this.query,
      onInitial: onInitial ?? this.onInitial,
      onFailure: onFailure ?? this.onFailure,
      onSuccess: onSuccess ?? this.onSuccess,
      onEmpty: onEmpty ?? this.onEmpty,
      buildList: buildList ?? this.buildList,
      customBuilder: customBuilder ?? this.customBuilder,
      buildItem: buildItem ?? this.buildItem,
      loadingWidget: loadingWidget ?? this.loadingWidget,
      maxReachedWidget: maxReachedWidget ?? this.maxReachedWidget,
      scrollController: scrollController ?? this.scrollController,
      physics: physics ?? this.physics,
      reverse: reverse,
      shrinkWrap: shrinkWrap,
    );
  }
}
